@@ -153,6 +153,34 @@ gst_gl_video_mixer_blend_function_get_type (void)
   return mixer_blend_function_type;
 }
 
+#define GST_TYPE_GL_VIDEO_MIXER_SIZING_POLICY (gst_gl_video_mixer_sizing_policy_get_type())
+static GType
+gst_gl_video_mixer_sizing_policy_get_type (void)
+{
+
+  static GType sizing_policy_type = 0;
+
+  static const GEnumValue sizing_polices[] = {
+    {GST_GL_VIDEO_MIXER_SIZING_POLICY_NONE,
+        "None: Image is scaled to fill configured destination rectangle without "
+          "padding or keeping the aspect ratio", "none"},
+    {GST_GL_VIDEO_MIXER_SIZING_POLICY_KEEP_ASPECT_RATIO,
+          "Keep Aspect Ratio: Image is scaled to fit destination rectangle "
+          "specified by GstGLVideoMixerPad:{xpos, ypos, width, height} "
+          "with preserved aspect ratio. The empty space of the resulting image "
+          "will be distributed in the destination rectangle according to the "
+          "GstGLVideoMixerPad:{xalign, yalign} values",
+        "keep-aspect-ratio"},
+    {0, NULL, NULL},
+  };
+
+  if (!sizing_policy_type) {
+    sizing_policy_type =
+        g_enum_register_static ("GstGLVideoMixerSizingPolicy", sizing_polices);
+  }
+  return sizing_policy_type;
+}
+
 #define DEFAULT_PAD_XPOS   0
 #define DEFAULT_PAD_YPOS   0
 #define DEFAULT_PAD_WIDTH  0
@@ -167,6 +195,9 @@ gst_gl_video_mixer_blend_function_get_type (void)
 #define DEFAULT_PAD_BLEND_FUNCTION_DST_RGB GST_GL_VIDEO_MIXER_BLEND_FUNCTION_ONE_MINUS_SRC_ALPHA
 #define DEFAULT_PAD_BLEND_FUNCTION_DST_ALPHA GST_GL_VIDEO_MIXER_BLEND_FUNCTION_ONE_MINUS_SRC_ALPHA
 #define DEFAULT_PAD_CROP 0
+#define DEFAULT_PAD_SIZING_POLICY GST_GL_VIDEO_MIXER_SIZING_POLICY_NONE
+#define DEFAULT_PAD_XALIGN 0.5
+#define DEFAULT_PAD_YALIGN 0.5
 
 enum
 {
@@ -192,6 +223,9 @@ enum
   PROP_INPUT_CROP_RIGHT,
   PROP_INPUT_CROP_TOP,
   PROP_INPUT_CROP_BOTTOM,
+  PROP_INPUT_SIZING_POLICY,
+  PROP_INPUT_XALIGN,
+  PROP_INPUT_YALIGN,
 };
 
 static void gst_gl_video_mixer_input_get_property (GObject * object,
@@ -374,6 +408,70 @@ gst_gl_video_mixer_input_class_init (GstGLVideoMixerInputClass * klass)
       g_param_spec_int ("crop-bottom", "Crop Bottom",
           "Crop bottom of the picture", 0, G_MAXINT, DEFAULT_PAD_CROP,
           G_PARAM_READWRITE | GST_PARAM_CONTROLLABLE | G_PARAM_STATIC_STRINGS));
+
+  /**
+   * GstGLVideoMixerInput:sizing-policy:
+   *
+   * Specifies sizing policy to use. Depending on selected sizing policy,
+   * scaled image might not fully cover the configured target rectangle area
+   * (e.g., "keep-aspect-ratio"). In that case, any uncovered area will be
+   * filled with background unless the uncovered area is drawn by other image.
+   *
+   * Since: 1.24
+   */
+  g_object_class_install_property (gobject_class, PROP_INPUT_SIZING_POLICY,
+      g_param_spec_enum ("sizing-policy", "Sizing policy",
+          "Sizing policy to use for image scaling",
+          GST_TYPE_GL_VIDEO_MIXER_SIZING_POLICY, DEFAULT_PAD_SIZING_POLICY,
+          G_PARAM_READWRITE | GST_PARAM_CONTROLLABLE | G_PARAM_STATIC_STRINGS));
+
+  /**
+   * GstGLVideoMixerInput:xalign:
+   *
+   * Defines the alignment of the input within the available horizontal space,
+   * relative to #GstGLVideoMixerPad:width and #GstGLVideoMixerPad:height.
+   * Values range from 0.0 (left) to 1.0 (right).
+   *
+   * The image is aligned in the available space as if the pivot point is
+   * matching the alignment. For example, setting the `xalign` property to 0.0
+   * will align the left edge of the image with the left edge of the bounding
+   * box; 0.5 aligns the horizontal center of the image with the horizontal
+   * center of the bounding box; 1.0 aligns the right edge of the image with the
+   * right edge of the bounding box; and so it goes.
+   *
+   * This property is only effective when #GstGLVideoMixerInput:sizing-policy
+   * is set to 'keep-aspect-ratio'.
+   *
+   * Since: 1.24
+   */
+  g_object_class_install_property (gobject_class, PROP_INPUT_XALIGN,
+      g_param_spec_double ("xalign", "X alignment",
+          "X alignment of the picture", 0.0, 1.0, DEFAULT_PAD_XALIGN,
+          G_PARAM_READWRITE | GST_PARAM_CONTROLLABLE | G_PARAM_STATIC_STRINGS));
+
+  /**
+   * GstGLVideoMixerInput:yalign:
+   *
+   * Defines the alignment of the input within the available vertical space,
+   * relative to #GstGLVideoMixerPad:width and #GstGLVideoMixerPad:height.
+   * Values range from 0.0 (top) to 1.0 (bottom).
+   *
+   * The image is aligned in the available space as if the pivot point is
+   * matching the alignment. For example, setting the `xalign` property to 0.0
+   * will align the left edge of the image with the left edge of the bounding
+   * box; 0.5 aligns the horizontal center of the image with the horizontal
+   * center of the bounding box; 1.0 aligns the right edge of the image with the
+   * right edge of the bounding box; and so it goes.
+   *
+   * This property is only effective when #GstGLVideoMixerInput:sizing-policy
+   * is set to 'keep-aspect-ratio'.
+   *
+   * Since: 1.24
+   */
+  g_object_class_install_property (gobject_class, PROP_INPUT_YALIGN,
+      g_param_spec_double ("yalign", "Y alignment",
+          "Y alignment of the picture", 0.0, 1.0, DEFAULT_PAD_YALIGN,
+          G_PARAM_READWRITE | GST_PARAM_CONTROLLABLE | G_PARAM_STATIC_STRINGS));
 }
 
 static void
@@ -402,6 +500,8 @@ gst_gl_video_mixer_input_dispose (GObject * object)
   GstGLVideoMixerInput *self = (GstGLVideoMixerInput *) object;
 
   gst_clear_object (&self->mixer_pad);
+
+  G_OBJECT_CLASS (gst_gl_video_mixer_input_parent_class)->dispose (object);
 }
 
 static GstGhostPad *
@@ -432,6 +532,9 @@ _create_video_mixer_input (GstGLMixerBin * self, GstPad * mixer_pad)
   ADD_BINDING (mixer_pad, input, "blend-constant-color-green");
   ADD_BINDING (mixer_pad, input, "blend-constant-color-blue");
   ADD_BINDING (mixer_pad, input, "blend-constant-color-alpha");
+  ADD_BINDING (mixer_pad, input, "sizing-policy");
+  ADD_BINDING (mixer_pad, input, "xalign");
+  ADD_BINDING (mixer_pad, input, "yalign");
 #undef ADD_BINDING
 
   input->mixer_pad = gst_object_ref (mixer_pad);
@@ -462,11 +565,23 @@ GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (glvideomixer, "glvideomixer",
 
 static void
 gst_gl_video_mixer_bin_init (GstGLVideoMixerBin * self)
+{
+}
+
+static void
+gst_gl_video_mixer_bin_constructed (GObject * self)
 {
   GstGLMixerBin *mix_bin = GST_GL_MIXER_BIN (self);
 
+  G_OBJECT_CLASS (gst_gl_video_mixer_bin_parent_class)->constructed (self);
+
   gst_gl_mixer_bin_finish_init_with_element (mix_bin,
-      g_object_new (GST_TYPE_GL_VIDEO_MIXER, NULL));
+      g_object_new (GST_TYPE_GL_VIDEO_MIXER,
+          "force-live", mix_bin->force_live,
+          "latency", mix_bin->latency,
+          "start-time-selection", mix_bin->start_time_selection,
+          "start-time", mix_bin->start_time,
+          "min-upstream-latency", mix_bin->min_upstream_latency, NULL));
 }
 
 static void
@@ -479,6 +594,7 @@ gst_gl_video_mixer_bin_class_init (GstGLVideoMixerBinClass * klass)
 
   mixer_class->create_input_pad = _create_video_mixer_input;
 
+  gobject_class->constructed = gst_gl_video_mixer_bin_constructed;
   gobject_class->set_property = gst_gl_video_mixer_bin_set_property;
   gobject_class->get_property = gst_gl_video_mixer_bin_get_property;
 
@@ -554,8 +670,6 @@ static gboolean gst_gl_video_mixer_propose_allocation (GstAggregator *
     agg, GstAggregatorPad * agg_pad, GstQuery * decide_query, GstQuery * query);
 static gboolean gst_gl_video_mixer_gl_start (GstGLBaseMixer * base_mix);
 static void gst_gl_video_mixer_gl_stop (GstGLBaseMixer * base_mix);
-static gboolean gst_gl_video_mixer_set_caps (GstGLMixer * mixer,
-    GstCaps * outcaps);
 
 static gboolean gst_gl_video_mixer_process_textures (GstGLMixer * mixer,
     GstGLMemory * out_tex);
@@ -629,7 +743,9 @@ struct _GstGLVideoMixerPad
   /* properties */
   gint xpos, ypos;
   gint width, height;
+  gdouble xalign, yalign;
   gdouble alpha;
+  GstGLVideoMixerSizingPolicy sizing_policy;
 
   GstGLVideoMixerBlendEquation blend_equation_rgb;
   GstGLVideoMixerBlendEquation blend_equation_alpha;
@@ -686,6 +802,9 @@ enum
   PROP_PAD_CROP_RIGHT,
   PROP_PAD_CROP_TOP,
   PROP_PAD_CROP_BOTTOM,
+  PROP_PAD_SIZING_POLICY,
+  PROP_PAD_XALIGN,
+  PROP_PAD_YALIGN,
 };
 
 static void
@@ -698,6 +817,9 @@ gst_gl_video_mixer_pad_init (GstGLVideoMixerPad * pad)
   pad->blend_function_src_alpha = DEFAULT_PAD_BLEND_FUNCTION_SRC_ALPHA;
   pad->blend_function_dst_rgb = DEFAULT_PAD_BLEND_FUNCTION_DST_RGB;
   pad->blend_function_dst_alpha = DEFAULT_PAD_BLEND_FUNCTION_DST_ALPHA;
+  pad->sizing_policy = DEFAULT_PAD_SIZING_POLICY;
+  pad->xalign = DEFAULT_PAD_XALIGN;
+  pad->yalign = DEFAULT_PAD_YALIGN;
   memset (pad->m_matrix, 0, sizeof (gfloat) * 4 * 4);
   pad->m_matrix[0] = 1.0;
   pad->m_matrix[5] = 1.0;
@@ -846,6 +968,70 @@ gst_gl_video_mixer_pad_class_init (GstGLVideoMixerPadClass * klass)
       g_param_spec_int ("crop-bottom", "Crop Bottom",
           "Crop bottom of the picture", 0, G_MAXINT, DEFAULT_PAD_CROP,
           G_PARAM_READWRITE | GST_PARAM_CONTROLLABLE | G_PARAM_STATIC_STRINGS));
+
+  /**
+   * GstGLVideoMixerPad:sizing-policy:
+   *
+   * Specifies sizing policy to use. Depending on selected sizing policy,
+   * scaled image might not fully cover the configured target rectangle area
+   * (e.g., "keep-aspect-ratio"). In that case, any uncovered area will be
+   * filled with background unless the uncovered area is drawn by other image.
+   *
+   * Since: 1.24
+   */
+  g_object_class_install_property (gobject_class, PROP_PAD_SIZING_POLICY,
+      g_param_spec_enum ("sizing-policy", "Sizing policy",
+          "Sizing policy to use for image scaling",
+          GST_TYPE_GL_VIDEO_MIXER_SIZING_POLICY, DEFAULT_PAD_SIZING_POLICY,
+          G_PARAM_READWRITE | GST_PARAM_CONTROLLABLE | G_PARAM_STATIC_STRINGS));
+
+  /**
+   * GstGLVideoMixerPad:xalign:
+   *
+   * Defines the alignment of the input within the available horizontal space,
+   * relative to #GstGLVideoMixerPad:width and #GstGLVideoMixerPad:height.
+   * Values range from 0.0 (left) to 1.0 (right).
+   *
+   * The image is aligned in the available space as if the pivot point is
+   * matching the alignment. For example, setting the `xalign` property to 0.0
+   * will align the left edge of the image with the left edge of the bounding
+   * box; 0.5 aligns the horizontal center of the image with the horizontal
+   * center of the bounding box; 1.0 aligns the right edge of the image with the
+   * right edge of the bounding box; and so it goes.
+   *
+   * This property is only effective when #GstGLVideoMixerInput:sizing-policy
+   * is set to 'keep-aspect-ratio'.
+   *
+   * Since: 1.24
+   */
+  g_object_class_install_property (gobject_class, PROP_PAD_XALIGN,
+      g_param_spec_double ("xalign", "X alignment",
+          "X alignment of the picture", 0.0, 1.0, DEFAULT_PAD_XALIGN,
+          G_PARAM_READWRITE | GST_PARAM_CONTROLLABLE | G_PARAM_STATIC_STRINGS));
+
+  /**
+   * GstGLVideoMixerPad:yalign:
+   *
+   * Defines the alignment of the input within the available vertical space,
+   * relative to #GstGLVideoMixerPad:width and #GstGLVideoMixerPad:height.
+   * Values range from 0.0 (top) to 1.0 (bottom).
+   *
+   * The image is aligned in the available space as if the pivot point is
+   * matching the alignment. For example, setting the `yalign` property to 0.0
+   * will align the top edge of the image with the top edge of the bounding box;
+   * 0.5 aligns the vertical center of the image with the vertical center of the
+   * bounding box; 1.0 aligns the bottom edge of the image with the bottom edge
+   * of the bounding box; and so it goes.
+   *
+   * This property is only effective when #GstGLVideoMixerInput:sizing-policy
+   * is set to 'keep-aspect-ratio'.
+   *
+   * Since: 1.24
+   */
+  g_object_class_install_property (gobject_class, PROP_PAD_YALIGN,
+      g_param_spec_double ("yalign", "Y alignment",
+          "Y alignment of the picture", 0.0, 1.0, DEFAULT_PAD_YALIGN,
+          G_PARAM_READWRITE | GST_PARAM_CONTROLLABLE | G_PARAM_STATIC_STRINGS));
 }
 
 static void
@@ -912,6 +1098,15 @@ gst_gl_video_mixer_pad_get_property (GObject * object, guint prop_id,
     case PROP_PAD_CROP_BOTTOM:
       g_value_set_int (value, pad->crop_bottom);
       break;
+    case PROP_PAD_SIZING_POLICY:
+      g_value_set_enum (value, pad->sizing_policy);
+      break;
+    case PROP_PAD_XALIGN:
+      g_value_set_double (value, pad->xalign);
+      break;
+    case PROP_PAD_YALIGN:
+      g_value_set_double (value, pad->yalign);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -1008,6 +1203,24 @@ gst_gl_video_mixer_pad_set_property (GObject * object, guint prop_id,
       pad->crop_bottom = val;
       break;
     }
+    case PROP_PAD_SIZING_POLICY:{
+      GstGLVideoMixerSizingPolicy val = g_value_get_enum (value);
+      pad->geometry_change |= val != pad->sizing_policy;
+      pad->sizing_policy = val;
+      break;
+    }
+    case PROP_PAD_XALIGN:{
+      gdouble val = g_value_get_double (value);
+      pad->geometry_change |= !G_APPROX_VALUE (val, pad->xalign, DBL_EPSILON);
+      pad->xalign = val;
+      break;
+    }
+    case PROP_PAD_YALIGN:{
+      gdouble val = g_value_get_double (value);
+      pad->geometry_change |= !G_APPROX_VALUE (val, pad->yalign, DBL_EPSILON);
+      pad->yalign = val;
+      break;
+    }
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -1094,6 +1307,7 @@ gst_gl_video_mixer_class_init (GstGLVideoMixerClass * klass)
       "Filter/Effect/Video/Compositor", "OpenGL video_mixer",
       "Matthew Waters <matthew@centricular.com>");
 
+  gst_gl_mixer_class_add_rgba_pad_templates (GST_GL_MIXER_CLASS (klass));
   gst_element_class_add_static_pad_template_with_gtype (element_class,
       &sink_factory, GST_TYPE_GL_VIDEO_MIXER_PAD);
 
@@ -1102,7 +1316,6 @@ gst_gl_video_mixer_class_init (GstGLVideoMixerClass * klass)
           GST_TYPE_GL_VIDEO_MIXER_BACKGROUND,
           DEFAULT_BACKGROUND, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
-  GST_GL_MIXER_CLASS (klass)->set_caps = gst_gl_video_mixer_set_caps;
   GST_GL_MIXER_CLASS (klass)->process_textures =
       gst_gl_video_mixer_process_textures;
 
@@ -1122,6 +1335,7 @@ gst_gl_video_mixer_class_init (GstGLVideoMixerClass * klass)
   gst_type_mark_as_plugin_api (GST_TYPE_GL_VIDEO_MIXER_PAD, 0);
   gst_type_mark_as_plugin_api (GST_TYPE_GL_VIDEO_MIXER_BLEND_EQUATION, 0);
   gst_type_mark_as_plugin_api (GST_TYPE_GL_VIDEO_MIXER_BLEND_FUNCTION, 0);
+  gst_type_mark_as_plugin_api (GST_TYPE_GL_VIDEO_MIXER_SIZING_POLICY, 0);
 }
 
 static void
@@ -1177,21 +1391,59 @@ gst_gl_video_mixer_propose_allocation (GstAggregator * agg,
   return TRUE;
 }
 
+static void
+align_rect (const GstVideoRectangle * src,
+    const GstVideoRectangle * dst, GstVideoRectangle * result, gdouble xalign,
+    gdouble yalign)
+{
+  gdouble src_ratio, dst_ratio;
+
+  g_return_if_fail (src->h != 0);
+  g_return_if_fail (dst->h != 0);
+
+  src_ratio = (gdouble) src->w / src->h;
+  dst_ratio = (gdouble) dst->w / dst->h;
+
+  if (src_ratio > dst_ratio) {
+    result->w = dst->w;
+    result->h = dst->w / src_ratio;
+    result->x = dst->x;
+    result->y = dst->y + (dst->h - result->h) * yalign;
+  } else if (src_ratio < dst_ratio) {
+    result->w = dst->h * src_ratio;
+    result->h = dst->h;
+    result->x = dst->x + (dst->w - result->w) * xalign;
+    result->y = dst->y;
+  } else {
+    result->x = dst->x;
+    result->y = dst->y;
+    result->w = dst->w;
+    result->h = dst->h;
+  }
+
+  GST_DEBUG ("source is %dx%d dest is %dx%d, result is %dx%d with x,y %dx%d",
+      src->w, src->h, dst->w, dst->h,
+      result->w, result->h, result->x, result->y);
+}
+
 static void
 _mixer_pad_get_output_size (GstGLVideoMixer * mix,
     GstGLVideoMixerPad * mix_pad, gint out_par_n, gint out_par_d, gint * width,
-    gint * height)
+    gint * height, gint * x_offset, gint * y_offset)
 {
   GstVideoAggregatorPad *vagg_pad = GST_VIDEO_AGGREGATOR_PAD (mix_pad);
   gint pad_width, pad_height;
   guint dar_n, dar_d;
 
+  *x_offset = 0;
+  *y_offset = 0;
+  *width = 0;
+  *height = 0;
+
   /* FIXME: Anything better we can do here? */
   if (!vagg_pad->info.finfo
       || vagg_pad->info.finfo->format == GST_VIDEO_FORMAT_UNKNOWN) {
     GST_DEBUG_OBJECT (mix_pad, "Have no caps yet");
-    *width = 0;
-    *height = 0;
     return;
   }
 
@@ -1228,12 +1480,83 @@ _mixer_pad_get_output_size (GstGLVideoMixer * mix,
       pad_height, dar_n, dar_d, GST_VIDEO_INFO_PAR_N (&vagg_pad->info),
       GST_VIDEO_INFO_PAR_D (&vagg_pad->info), out_par_n, out_par_d);
 
-  if (pad_height % dar_n == 0) {
-    pad_width = gst_util_uint64_scale_int (pad_height, dar_n, dar_d);
-  } else if (pad_width % dar_d == 0) {
-    pad_height = gst_util_uint64_scale_int (pad_width, dar_d, dar_n);
-  } else {
-    pad_width = gst_util_uint64_scale_int (pad_height, dar_n, dar_d);
+  switch (mix_pad->sizing_policy) {
+    case GST_GL_VIDEO_MIXER_SIZING_POLICY_NONE:
+      /* Pick either height or width, whichever is an integer multiple of the
+       * display aspect ratio. However, prefer preserving the height to account
+       * for interlaced video. */
+      if (pad_height % dar_n == 0) {
+        pad_width = gst_util_uint64_scale_int (pad_height, dar_n, dar_d);
+      } else if (pad_width % dar_d == 0) {
+        pad_height = gst_util_uint64_scale_int (pad_width, dar_d, dar_n);
+      } else {
+        pad_width = gst_util_uint64_scale_int (pad_height, dar_n, dar_d);
+      }
+      break;
+    case GST_GL_VIDEO_MIXER_SIZING_POLICY_KEEP_ASPECT_RATIO:
+    {
+      gint from_dar_n, from_dar_d, to_dar_n, to_dar_d, num, den;
+
+      /* Calculate DAR again with actual video size */
+      if (!gst_util_fraction_multiply (GST_VIDEO_INFO_WIDTH (&vagg_pad->info),
+              GST_VIDEO_INFO_HEIGHT (&vagg_pad->info),
+              GST_VIDEO_INFO_PAR_N (&vagg_pad->info),
+              GST_VIDEO_INFO_PAR_D (&vagg_pad->info), &from_dar_n,
+              &from_dar_d)) {
+        from_dar_n = from_dar_d = -1;
+      }
+
+      if (!gst_util_fraction_multiply (pad_width, pad_height,
+              out_par_n, out_par_d, &to_dar_n, &to_dar_d)) {
+        to_dar_n = to_dar_d = -1;
+      }
+
+      if (from_dar_n != to_dar_n || from_dar_d != to_dar_d) {
+        /* Calculate new output resolution */
+        if (from_dar_n != -1 && from_dar_d != -1
+            && gst_util_fraction_multiply (from_dar_n, from_dar_d,
+                out_par_d, out_par_n, &num, &den)) {
+          GstVideoRectangle src_rect, dst_rect;
+          GstVideoRectangle rst_rect = { 0, 0, 0, 0 };
+
+          src_rect.h = gst_util_uint64_scale_int (pad_width, den, num);
+          if (src_rect.h == 0) {
+            pad_width = 0;
+            pad_height = 0;
+            break;
+          }
+
+          src_rect.x = src_rect.y = 0;
+          src_rect.w = pad_width;
+
+          dst_rect.x = dst_rect.y = 0;
+          dst_rect.w = pad_width;
+          dst_rect.h = pad_height;
+
+          /* Scale rect to be centered in destination rect */
+          align_rect (&src_rect, &dst_rect, &rst_rect, mix_pad->xalign,
+              mix_pad->yalign);
+
+          GST_LOG_OBJECT (mix_pad,
+              "Re-calculated size %dx%d -> %dx%d (x-offset %d, y-offset %d)",
+              pad_width, pad_height, rst_rect.w, rst_rect.h, rst_rect.x,
+              rst_rect.h);
+
+          *x_offset = rst_rect.x;
+          *y_offset = rst_rect.y;
+          pad_width = rst_rect.w;
+          pad_height = rst_rect.h;
+        } else {
+          GST_WARNING_OBJECT (mix_pad, "Failed to calculate output size");
+
+          *x_offset = 0;
+          *y_offset = 0;
+          pad_width = 0;
+          pad_height = 0;
+        }
+      }
+      break;
+    }
   }
 
   *width = pad_width;
@@ -1303,18 +1626,20 @@ _fixate_caps (GstAggregator * agg, GstCaps * caps)
     GstGLVideoMixerPad *mixer_pad = GST_GL_VIDEO_MIXER_PAD (vaggpad);
     gint this_width, this_height;
     gint width, height;
+    gint offset_x, offset_y;
     gint fps_n, fps_d;
     gdouble cur_fps;
 
     fps_n = GST_VIDEO_INFO_FPS_N (&vaggpad->info);
     fps_d = GST_VIDEO_INFO_FPS_D (&vaggpad->info);
-    _mixer_pad_get_output_size (mix, mixer_pad, par_n, par_d, &width, &height);
+    _mixer_pad_get_output_size (mix, mixer_pad, par_n, par_d, &width, &height,
+        &offset_x, &offset_y);
 
     if (width == 0 || height == 0)
       continue;
 
-    this_width = width + MAX (mixer_pad->xpos, 0);
-    this_height = height + MAX (mixer_pad->ypos, 0);
+    this_width = width + MAX (mixer_pad->xpos + offset_x, 0);
+    this_height = height + MAX (mixer_pad->ypos + offset_y, 0);
 
     if (best_width < this_width)
       best_width = this_width;
@@ -1401,24 +1726,27 @@ src_pad_mouse_event (GstElement * element, GstPad * pad, gpointer user_data)
 {
   GstGLVideoMixer *mix = GST_GL_VIDEO_MIXER (element);
   GstGLVideoMixerPad *mix_pad = GST_GL_VIDEO_MIXER_PAD (pad);
-  GstCaps *caps = gst_pad_get_current_caps (pad);
-  GstStructure *event_st, *caps_st;
-  gint par_n = 1, par_d = 1;
+  GstVideoAggregatorPad *vagg_pad = GST_VIDEO_AGGREGATOR_PAD (mix_pad);
+  GstStructure *event_st;
   gdouble event_x, event_y;
   GstVideoRectangle rect;
 
+  if (!vagg_pad->info.finfo
+      || vagg_pad->info.finfo->format == GST_VIDEO_FORMAT_UNKNOWN) {
+    GST_DEBUG_OBJECT (mix_pad, "Have no caps yet");
+    return TRUE;
+  }
+
   event_st =
       gst_structure_copy (gst_event_get_structure (GST_EVENT_CAST (user_data)));
-  caps_st = gst_structure_copy (gst_caps_get_structure (caps, 0));
-
   gst_structure_get (event_st, "pointer_x", G_TYPE_DOUBLE, &event_x,
       "pointer_y", G_TYPE_DOUBLE, &event_y, NULL);
 
   /* Find output rectangle of this pad */
-  gst_structure_get_fraction (caps_st, "pixel-aspect-ratio", &par_n, &par_d);
-  _mixer_pad_get_output_size (mix, mix_pad, par_n, par_d, &(rect.w), &(rect.h));
-  rect.x = mix_pad->xpos;
-  rect.y = mix_pad->ypos;
+  _mixer_pad_get_output_size (mix, mix_pad, vagg_pad->info.par_n,
+      vagg_pad->info.par_d, &(rect.w), &(rect.h), &(rect.x), &(rect.y));
+  rect.x += mix_pad->xpos;
+  rect.y += mix_pad->ypos;
 
   /* Translate coordinates and send event if it lies in this rectangle */
   if (is_point_contained (rect, event_x, event_y)) {
@@ -1437,7 +1765,6 @@ src_pad_mouse_event (GstElement * element, GstPad * pad, gpointer user_data)
     gst_structure_free (event_st);
   }
 
-  gst_structure_free (caps_st);
   return TRUE;
 }
 
@@ -1472,17 +1799,6 @@ gst_gl_video_mixer_src_event (GstAggregator * agg, GstEvent * event)
   return GST_AGGREGATOR_CLASS (parent_class)->src_event (agg, event);
 }
 
-static gboolean
-gst_gl_video_mixer_set_caps (GstGLMixer * mixer, GstCaps * outcaps)
-{
-  GstGLVideoMixer *video_mixer = GST_GL_VIDEO_MIXER (mixer);
-
-  /* need reconfigure output geometry */
-  video_mixer->output_geo_change = TRUE;
-
-  return TRUE;
-}
-
 static void
 gst_gl_video_mixer_gl_stop (GstGLBaseMixer * base_mix)
 {
@@ -1501,6 +1817,8 @@ gst_gl_video_mixer_gl_start (GstGLBaseMixer * base_mix)
 {
   GstGLVideoMixer *video_mixer = GST_GL_VIDEO_MIXER (base_mix);
 
+  video_mixer->output_geo_change = TRUE;
+
   if (!video_mixer->shader) {
     gchar *frag_str = g_strdup_printf ("%s%s",
         gst_gl_shader_string_get_highest_precision (base_mix->context,
@@ -1521,9 +1839,12 @@ static void
 _video_mixer_process_gl (GstGLContext * context, GstGLVideoMixer * video_mixer)
 {
   GstGLMixer *mixer = GST_GL_MIXER (video_mixer);
+  GstGLFramebuffer *fbo = gst_gl_mixer_get_framebuffer (mixer);
 
-  gst_gl_framebuffer_draw_to_texture (mixer->fbo, video_mixer->out_tex,
+  gst_gl_framebuffer_draw_to_texture (fbo, video_mixer->out_tex,
       gst_gl_video_mixer_callback, video_mixer);
+
+  gst_clear_object (&fbo);
 }
 
 static gboolean
@@ -1840,6 +2161,7 @@ gst_gl_video_mixer_callback (gpointer stuff)
     if (video_mixer->output_geo_change
         || pad->geometry_change || !pad->vertex_buffer) {
       gint pad_width, pad_height;
+      gint pad_offset_x, pad_offset_y;
       gfloat w, h;
       /* *INDENT-OFF* */
       gfloat v_vertices[] = {
@@ -1852,7 +2174,8 @@ gst_gl_video_mixer_callback (gpointer stuff)
 
       _mixer_pad_get_output_size (video_mixer, pad,
           GST_VIDEO_INFO_PAR_N (&vagg->info),
-          GST_VIDEO_INFO_PAR_D (&vagg->info), &pad_width, &pad_height);
+          GST_VIDEO_INFO_PAR_D (&vagg->info),
+          &pad_width, &pad_height, &pad_offset_x, &pad_offset_y);
 
       w = ((gfloat) pad_width / (gfloat) out_width);
       h = ((gfloat) pad_height / (gfloat) out_height);
@@ -1860,9 +2183,11 @@ gst_gl_video_mixer_callback (gpointer stuff)
       pad->m_matrix[0] = w;
       pad->m_matrix[5] = h;
       pad->m_matrix[12] =
-          2. * (gfloat) pad->xpos / (gfloat) out_width - (1. - w);
+          2. * (gfloat) (pad->xpos + pad_offset_x) / (gfloat) out_width - (1. -
+          w);
       pad->m_matrix[13] =
-          2. * (gfloat) pad->ypos / (gfloat) out_height - (1. - h);
+          2. * (gfloat) (pad->ypos + pad_offset_y) / (gfloat) out_height - (1. -
+          h);
 
       v_vertices[0 * 5 + 3] = v_vertices[3 * 5 + 3] =
           pad->crop_left ? ((gfloat) pad->crop_left) /
