@@ -24,8 +24,18 @@
 
 #include <gst/gst.h>
 
-#ifndef GST_GL_API
-#define GST_GL_API GST_EXPORT
+#ifdef BUILDING_GST_GL
+#define GST_GL_API GST_API_EXPORT         /* from config.h */
+#else
+#define GST_GL_API GST_API_IMPORT
+#endif
+
+#ifndef GST_DISABLE_DEPRECATED
+#define GST_GL_DEPRECATED GST_GL_API
+#define GST_GL_DEPRECATED_FOR(f) GST_GL_API
+#else
+#define GST_GL_DEPRECATED G_DEPRECATED GST_GL_API
+#define GST_GL_DEPRECATED_FOR(f) G_DEPRECATED_FOR(f) GST_GL_API
 #endif
 
 #endif /* __GST_GL_PRELUDE_H__ */
